@@ -1,6 +1,6 @@
 /* ncftp.h
  *
- * Copyright (c) 1996-2008 Mike Gleason, NcFTP Software.
+ * Copyright (c) 1996-2025 Mike Gleason, NcFTP Software.
  * All rights reserved.
  *
  */
@@ -13,27 +13,34 @@ extern "C"
 {
 #endif	/* __cplusplus */
 
-#define kLibraryVersion "@(#) LibNcFTP 3.2.3 (August 18, 2008)"
+#define kLibraryVersion "@(#) LibNcFTP 3.2.8 (March 8, 2025)"
 
 /* This is used to verify validty of the data passed in. */
-#define kLibraryMagic "LibNcFTP 3.2.3"
+#define kLibraryMagic "LibNcFTP 3.2.8"
+#define kLibraryMagicLen 14
 
 #if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 	/* Includes for Windows */
 #	pragma once
-#	pragma warning(disable : 4127)	// warning C4127: conditional expression is constant
-#	pragma warning(disable : 4100)	// warning C4100: 'lpReserved' : unreferenced formal parameter
-#	pragma warning(disable : 4514)	// warning C4514: unreferenced inline function has been removed
-#	pragma warning(disable : 4115)	// warning C4115: '_RPC_ASYNC_STATE' : named type definition in parentheses
-#	pragma warning(disable : 4201)	// warning C4201: nonstandard extension used : nameless struct/union
-#	pragma warning(disable : 4214)	// warning C4214: nonstandard extension used : bit field types other than int
-#	pragma warning(disable : 4115)	// warning C4115: 'IRpcStubBuffer' : named type definition in parentheses
-#	pragma warning(disable : 4711)	// warning C4711: function selected for automatic inline expansion
+#	ifndef __MINGW32__
+#		pragma warning(disable : 4127)	// warning C4127: conditional expression is constant
+#		pragma warning(disable : 4100)	// warning C4100: 'lpReserved' : unreferenced formal parameter
+#		pragma warning(disable : 4514)	// warning C4514: unreferenced inline function has been removed
+#		pragma warning(disable : 4115)	// warning C4115: '_RPC_ASYNC_STATE' : named type definition in parentheses
+#		pragma warning(disable : 4201)	// warning C4201: nonstandard extension used : nameless struct/union
+#		pragma warning(disable : 4214)	// warning C4214: nonstandard extension used : bit field types other than int
+#		pragma warning(disable : 4115)	// warning C4115: 'IRpcStubBuffer' : named type definition in parentheses
+#		pragma warning(disable : 4711)	// warning C4711: function selected for automatic inline expansion
+#	endif
+	/* We now try for at least Windows 2000 compatibility (0x0500).
+	 * The code will still work on older systems, though.
+	 * Prior versions used 0x0400 instead.
+	 */
 #	ifndef WINVER
-#		define WINVER 0x0400
+#		define WINVER 0x0500
 #	endif
 #	ifndef _WIN32_WINNT
-#		define _WIN32_WINNT 0x0400
+#		define _WIN32_WINNT 0x0500
 #	endif
 #	ifndef _INC_WINDOWS
 #		include <windows.h>	/* includes <winsock2.h> if _WIN32_WINNT >= 0x400 */
@@ -165,17 +172,6 @@ extern "C"
 #	endif
 #else
 	/* Includes for UNIX */
-#	if ((defined(__linux__)) && (! defined(_LARGEFILE64_SOURCE)) && (! defined(NO_LARGEFILE64_SOURCE)) && (! defined(_LARGEFILE_SOURCE)) && (! defined(_FILE_OFFSET_BITS)))
-		/* Define _LARGEFILE64_SOURCE or NO_LARGEFILE64_SOURCE to
-		 * quiet this warning.  Be sure to define this for ALL
-		 * code that uses LibNcFTP.
-		 */
-#		warning "<ncftp.h> is #defining _LARGEFILE64_SOURCE for you"
-#		define _LARGEFILE64_SOURCE 1
-#		define   Stat    stat64
-#		define   Lstat   lstat64
-#		define   Fstat   fstat64
-#	endif
 #	include <unistd.h>
 #	include <sys/types.h>
 #	include <sys/stat.h>
@@ -227,6 +223,11 @@ typedef struct FTPLine {
 	char *line;
 } FTPLine;
 
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wpadded"
+#endif
+
 typedef struct FTPLineList {
 	FTPLinePtr first, last;
 	int nLines;
@@ -241,8 +242,8 @@ typedef struct Response {
 	int hadEof;
 } Response, *ResponsePtr;
 
-#if USE_SIO && !defined(_SReadlineInfo_)
-#define _SReadlineInfo_ 1
+#if USE_SIO && !defined(defined_SReadlineInfo)
+#define defined_SReadlineInfo 1
 typedef struct SReadlineInfo {
 	char *buf;		/* Pointer to beginning of buffer. */
 	char *bufPtr;		/* Pointer to current position in buffer. */
@@ -255,6 +256,10 @@ typedef struct SReadlineInfo {
 	int requireEOLN;	/* When buffer is full, continue reading and discarding until \n? */
 } SReadlineInfo;
 #endif
+    
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif
 
 typedef struct FTPLibraryInfo {
 	char magic[16];				/* Don't modify this field. */
@@ -276,20 +281,21 @@ typedef void (*FTPGetPassphraseProc)(const FTPCIPtr, FTPLineListPtr pwPrompt, ch
 typedef struct FTPConnectionInfo {
 	char magic[16];				/* Don't modify this field. */
 	
-	char host[64];				/* REQUIRED input parameter. */
-	char user[64];				/* OPTIONAL input parameter. */
-	char pass[64];				/* OPTIONAL input parameter. */
+	char host[128];				/* REQUIRED input parameter. */
+	char user[128];				/* OPTIONAL input parameter. */
+	char pass[256];				/* OPTIONAL input parameter. */
 	char acct[64];				/* OPTIONAL input parameter. */
 	unsigned int passIsEmpty;		/* OPTIONAL input parameter. */
 	unsigned int port;			/* OPTIONAL input parameter. */
 
 	int errNo;				/* You may modify this if you want. */
 	char lastFTPCmdResultStr[128];		/* You may modify this if you want. */
+    int lastFTPCmdResultNum;		/* You may modify this if you want. */
 	FTPLineList lastFTPCmdResultLL;		/* Use, but do not modify. */
-	int lastFTPCmdResultNum;		/* You may modify this if you want. */
 
 	FILE *debugLog;				/* OPTIONAL input parameter. */
 	FTPLogProc debugLogProc;		/* OPTIONAL input parameter. */
+    int debugTimestamping;			/* OPTIONAL input parameter. */
 
 	unsigned int xferTimeout;		/* OPTIONAL input parameter. */
 	unsigned int connTimeout;		/* OPTIONAL input parameter. */
@@ -328,7 +334,6 @@ typedef struct FTPConnectionInfo {
 	/* Used during transfers; valid during progress meter updates. */
 	FTPProgressMeterProc progress;		/* OPTIONAL input parameter. */
 	longest_int bytesTransferred;		/* Do not modify this field. */
-	int useProgressMeter;			/* Used internally. */
 	struct timeval t0;			/* Do not modify this field. */
 	double sec;				/* Do not modify this field. */
 	double secLeft;				/* Do not modify this field. */
@@ -339,6 +344,7 @@ typedef struct FTPConnectionInfo {
 	time_t nextProgressUpdate;		/* Do not modify this field. */
 	const char *rname;			/* Do not modify this field. */
 	const char *lname;			/* Do not modify this field. */
+    int useProgressMeter;			/* Used internally. */
 	int stalled;				/* Do not modify this field. */
 	int dataTimedOut;			/* Do not modify this field. */
 	int cancelXfer;				/* You may modify this. */
@@ -349,10 +355,13 @@ typedef struct FTPConnectionInfo {
 	char ip[32];				/* Do not modify this field. */
 
 	char *startingWorkingDirectory;		/* Use, but do not modify. */
+	char *currentWorkingDirectory;		/* Use, but do not modify. */
+	size_t currentWorkingDirectorySize;	/* Use, but do not modify. */
 
 	int connected;				/* Do not modify this field. */
 	int loggedIn;				/* Do not modify this field. */
 	int curTransferType;			/* Do not modify this field. */
+    int reserved2;
 	longest_int startPoint;			/* Do not modify this field. */
 	FILE *errLog;				/* DEPRECATED input parameter. */
 	FTPLogProc errLogProc;			/* DEPRECATED input parameter. */
@@ -362,7 +371,10 @@ typedef struct FTPConnectionInfo {
 	int hasSIZE;				/* Do not modify this field. */
 	int hasMDTM;				/* Do not modify this field. */
 	int hasMDTM_set;			/* Do not modify this field. */
+	int hasMFMT;				/* Do not modify this field. */
+	int hasMFF;				/* Do not modify this field. */
 	int hasREST;				/* Do not modify this field. */
+	int hasAPPE;				/* You may modify this. */
 	int hasNLST_a;				/* Do not modify this field. */
 	int hasNLST_d;				/* Do not modify this field. */
 	int hasFEAT;				/* Do not modify this field. */
@@ -379,6 +391,7 @@ typedef struct FTPConnectionInfo {
 	int hasSITE_SBUFSIZ;			/* Do not modify this field. */
 	int hasSITE_SBUFSZ;			/* Do not modify this field. */
 	int hasSITE_BUFSIZE;			/* Do not modify this field. */
+	int hasRETR_tar;			/* Do not modify this field. */
 	int mlsFeatures;			/* Do not modify this field. */
 	int STATfileParamWorks;			/* Do not modify this field. */
 	int NLSTfileParamWorks;			/* Do not modify this field. */
@@ -388,9 +401,9 @@ typedef struct FTPConnectionInfo {
 	struct sockaddr_in ourCtlAddr;		/* Do not modify this field. */
 	struct sockaddr_in ourDataAddr;		/* Do not modify this field. */
 	int netMode;				/* Do not use or modify. */
+    int doAllocBuf;				/* Do not modify this field. */
 	char *buf;				/* Do not modify this field. */
 	size_t bufSize;				/* Do not modify this field. */
-	int doAllocBuf;				/* Do not modify this field. */
 	FILE *cin;				/* Do not use or modify. */
 	FILE *cout;				/* Do not use or modify. */
 	int ctrlSocketR;			/* You may use but not modify/close. */
@@ -407,7 +420,8 @@ typedef struct FTPConnectionInfo {
 
 	int numDials;				/* Do not modify. */
 	int totalDials;				/* Do not modify. */
-	struct timeval initTime;		/* Do not modify. */
+    int doNotGetStartingWorkingDirectory;	/* You may modify this. */
+    struct timeval initTime;		/* Do not modify. */
 	struct timeval startTime;		/* Do not modify. */
 	struct timeval connectTime;		/* Do not modify. */
 	struct timeval loginTime;		/* Do not modify. */
@@ -417,8 +431,7 @@ typedef struct FTPConnectionInfo {
 	int numDownloads;			/* Do not modify. */
 	int numUploads;				/* Do not modify. */
 	int numListings;			/* Do not modify. */
-
-	int doNotGetStartingWorkingDirectory;	/* You may modify this. */
+    int reserved3;
 	char textEOLN[4];			/* Set automatically per platform. */
 	int asciiTranslationMode;		/* You may modify this. */
 	const char *manualOverrideFeatures;	/* You may modify this. */
@@ -431,16 +444,26 @@ typedef struct FTPConnectionInfo {
 	 */
 	struct sockaddr_in clientKnownExternalAddr;	
 
+	struct sockaddr_in preferredLocalAddr;	/* You may modify this. */
 	int maxNumberOfSuccessivePASVAttempts;	/* You may modify this. */
+	int recentMdtmHeuristic;		/* You may modify this. */
+	time_t recentMdtmThreshold;		/* You may modify this. */
+	time_t timeBetweenUtimeBlocks;		/* You may modify this. */
+    int utimeBlocks;			/* You may modify this. */
+	int useSendfile;			/* You may modify this. */
+	int usingSendfile;			/* Do not use or modify. */
 
 #if USE_SIO
-	char srlBuf[768];
+    int srlReserved;
+	char srlBuf[1024];
 	SReadlineInfo ctrlSrl;			/* Do not use or modify. */
 #endif
-	int iUser;				/* Scratch integer field you can use. */
+    int iUser;				/* Scratch integer field you can use. */
+    int iUser2;				/* Scratch integer field you can use. */
 	void *pUser;				/* Scratch pointer field you can use. */
 	longest_int llUser;			/* Scratch long long field you can use. */
 	int reserved[32];			/* Do not use or modify. */
+	char tailMagic[16];			/* Do not use or modify. */
 } FTPConnectionInfo;
 
 typedef struct FTPFileInfo *FTPFileInfoPtr, **FTPFileInfoVec;
@@ -452,10 +475,10 @@ typedef struct FTPFileInfo {
 	char *lname;
 	char *plug;	/* permissions, links, user, group */
 	int type;
+    int mode;	/* Only set by UnMlsD() */
 	time_t mdtm;
 	longest_int size;
 	size_t relnameLen;
-	int mode;	/* Only set by UnMlsD() */
 } FTPFileInfo;
 
 typedef struct FTPFileInfoList {
@@ -466,15 +489,16 @@ typedef struct FTPFileInfoList {
 	int nFileInfos;
 	int sortKey;
 	int sortOrder;
+    int reserved;
 } FTPFileInfoList, *FTPFileInfoListPtr;
 
 /* Used with UnMlsT() */
 typedef struct MLstItem{
 	char fname[512];
 	char linkto[512];
-	int ftype;
 	longest_int fsize;
 	time_t ftime;
+    int ftype;
 	int mode;		/* "UNIX.mode" fact */
 	int uid;		/* "UNIX.uid" fact */
 	int gid;		/* "UNIX.gid" fact */
@@ -514,6 +538,8 @@ typedef struct MLstItem{
 #	define kDefaultFTPBufSize	32768
 #endif
 
+#define kDefaultPathBufSize		4096
+
 #ifdef USE_SIO
 /* This version of the library can handle timeouts without
  * a user-installed signal handler.
@@ -733,6 +759,8 @@ typedef int (*FTPConfirmResumeUploadProc)(
 #	define TVFSPathToLocalPath(s)
 #	define LocalPathToTVFSPath(s)
 #endif
+#define LocalPathBuild(dst,dsize,dlen,pdir,fn) PathBuild(dst, dsize, dlen, LOCAL_PATH_DELIM, pdir, fn)
+#define TVFSPathBuild(dst,dsize,dlen,pdir,fn) PathBuild(dst, dsize, dlen, '/', pdir, fn)
 
 #if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 typedef struct dirent {
@@ -775,9 +803,9 @@ int WinStat64(const char *const path, struct WinStat64 *const stp);
 
 /* The following block may be changed by configure script */
 #ifndef Stat
-#define Stat stat64    /* set by configure at 2008-07-15 10:17:24 */
-#define Lstat lstat64    /* set by configure at 2008-07-15 10:17:24 */
-#define Fstat fstat64    /* set by configure at 2008-07-15 10:17:24 */
+#define Stat stat
+#define Lstat lstat
+#define Fstat fstat
 #endif
 
 #define kFtwNoAutoGrowButContinue (-1)
@@ -790,7 +818,7 @@ typedef struct FtwInfo *FtwInfoPtr;
 typedef int (*FtwProc)(const FtwInfoPtr ftwip);
 
 typedef struct FtwInfo {
-	unsigned int init;
+	unsigned int magic, init;
 	FtwProc proc;
 	char *curPath;
 	size_t curPathLen;
@@ -798,14 +826,16 @@ typedef struct FtwInfo {
 	size_t startPathLen;
 	char *curFile;
 	size_t curFileLen;
-	int curType;
 	struct Stat curStat;
+    int curType;
 	int noAutoMallocAndFree;
 	int dirSeparator;
 	char rootDir[4];
 	int autoGrow;
+	int isBaseDir;
 	size_t depth;
 	size_t maxDepth;
+	size_t maxDepthAllowed;
 	size_t numDirs;
 	size_t numFiles;
 	size_t numLinks;
@@ -843,10 +873,10 @@ typedef struct GetoptInfo {
 #endif
 
 extern const char gLibNcFTPVersion[];
-extern int gLibNcFTP_Uses_Me_To_Quiet_Variable_Unused_Warnings;
+extern const char *gLibNcFTP_Uses_Me_To_Quiet_Variable_Unused_Warnings;
 
 #if (defined(__APPLE_CC__)) && (__APPLE_CC__ < 10000)
-#	define LIBNCFTP_USE_VAR(a) gLibNcFTP_Uses_Me_To_Quiet_Variable_Unused_Warnings = (a == 0)
+#	define LIBNCFTP_USE_VAR(a) gLibNcFTP_Uses_Me_To_Quiet_Variable_Unused_Warnings = (const char *) (&a)
 #	ifndef UNUSED
 #		define UNUSED(a) a
 #	endif
@@ -861,7 +891,7 @@ extern int gLibNcFTP_Uses_Me_To_Quiet_Variable_Unused_Warnings;
 #	endif
 #	define LIBNCFTP_USE_VAR(a)
 #else
-#	define LIBNCFTP_USE_VAR(a) gLibNcFTP_Uses_Me_To_Quiet_Variable_Unused_Warnings = (a == 0)
+#	define LIBNCFTP_USE_VAR(a) gLibNcFTP_Uses_Me_To_Quiet_Variable_Unused_Warnings = (const char *) (&a)
 #	ifndef UNUSED
 #		define UNUSED(a) a
 #	endif
@@ -910,8 +940,8 @@ int FTPOpenHost(const FTPCIPtr cip);
 int FTPOpenHostNoLogin(const FTPCIPtr cip);
 void FTPPerror(const FTPCIPtr cip, const int err, const int eerr, const char *const s1, const char *const s2);
 int FTPPutFileFromMemory(const FTPCIPtr cip, const char *volatile dstfile, const char *volatile src, const size_t srcLen, const int appendflag);
-int FTPPutFiles3(const FTPCIPtr cip, const char *const pattern, const char *const dstdir1, const int recurse, const int doGlob, const int xtype, int appendflag, const char *const tmppfx, const char *const tmpsfx, const int resumeflag, const int deleteflag, const FTPConfirmResumeUploadProc resumeProc, int UNUSED(reserved));
-int FTPPutOneFile3(const FTPCIPtr cip, const char *const file, const char *const dstfile, const int xtype, const int fdtouse, const int appendflag, const char *const tmppfx, const char *const tmpsfx, const int resumeflag, const int deleteflag, const FTPConfirmResumeUploadProc resumeProc, int UNUSED(reserved));
+int FTPPutOneFile4(const FTPCIPtr cip, const char *const file, const char *const dstfile, const int xtype, const int fdtouse, const int appendflag, const char *const tmppfx, const char *const tmpsfx, const int resumeflag, const int deleteflag, const FTPConfirmResumeUploadProc resumeProc, const time_t batchStartTime, const time_t origLmtime); 
+int FTPPutFiles4(const FTPCIPtr cip, const char *const pattern, const char *const dstdir1, const int recurse, const int doGlob, const int xtype, const int appendflag, const char *const tmppfx, const char *const tmpsfx, const int resumeflag, const int deleteflag, const FTPConfirmResumeUploadProc resumeProc, const time_t batchStartTime, const time_t origLmtime);
 int FTPReadLoginConfigFile(FTPCIPtr cip, const char *const fn);
 int FTPRemoteGlob(FTPCIPtr cip, FTPLineListPtr fileList, const char *pattern, int doGlob);
 int FTPRename(const FTPCIPtr cip, const char *const oldname, const char *const newname);
@@ -954,7 +984,11 @@ void GetHomeDir(char *const dst, const size_t size);
 void GetTmpDir(char *const dst, const size_t size);
 void GetUsrName(char *, size_t);
 void Scramble(unsigned char *dst, size_t dsize, unsigned char *src, char *key);
-double FTPDuration(struct timeval *t0);
+#ifndef tvcopy
+#	define tvcopy(dst,src) dst.tv_sec = src.tv_sec; dst.tv_usec = src.tv_usec
+#endif
+double FTPDuration(struct timeval *const t0);
+double FTPDuration2(struct timeval *const xt0, struct timeval *const xt1);
 struct tm *Gmtime(time_t t, struct tm *const tp);
 struct tm *Localtime(time_t t, struct tm *const tp);
 time_t GetUTCOffset(const int mon, const int mday);
@@ -973,6 +1007,7 @@ void StrRemoveTrailingLocalPathDelim(char *dst);
 void TVFSPathToLocalPath(char *dst);
 void LocalPathToTVFSPath(char *dst);
 #endif
+int PathBuild(char *const dst, const size_t dsize, size_t *const dResultingLength, int dirSep, const char *const parentDir, const char *const fileName);
 #if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 int gettimeofday(struct timeval *const tp, void *junk);
 void WinSleep(unsigned int seconds);
@@ -993,8 +1028,10 @@ int FTPGetOneFileAscii(const FTPCIPtr cip, const char *const file, const char *c
 int FTPGetFilesAscii(const FTPCIPtr cip, const char *const pattern, const char *const dstdir, const int recurse, const int doGlob);
 int FTPPutOneFile(const FTPCIPtr cip, const char *const file, const char *const dstfile);
 int FTPPutOneFile2(const FTPCIPtr cip, const char *const file, const char *const dstfile, const int xtype, const int fdtouse, const int appendflag, const char *const tmppfx, const char *const tmpsfx);
+int FTPPutOneFile3(const FTPCIPtr cip, const char *const file, const char *const dstfile, const int xtype, const int fdtouse, const int appendflag, const char *const tmppfx, const char *const tmpsfx, const int resumeflag, const int deleteflag, const FTPConfirmResumeUploadProc resumeProc, int UNUSED(reserved));
 int FTPPutFiles(const FTPCIPtr cip, const char *const pattern, const char *const dstdir, const int recurse, const int doGlob);
 int FTPPutFiles2(const FTPCIPtr cip, const char *const pattern, const char *const dstdir, const int recurse, const int doGlob, const int xtype, const int appendflag, const char *const tmppfx, const char *const tmpsfx);
+int FTPPutFiles3(const FTPCIPtr cip, const char *const pattern, const char *const dstdir1, const int recurse, const int doGlob, const int xtype, int appendflag, const char *const tmppfx, const char *const tmpsfx, const int resumeflag, const int deleteflag, const FTPConfirmResumeUploadProc resumeProc, int UNUSED(reserved));
 int FTPPutOneFileAscii(const FTPCIPtr cip, const char *const file, const char *const dstfile);
 int FTPPutFilesAscii(const FTPCIPtr cip, const char *const pattern, const char *const dstdir, const int recurse, const int doGlob);
 
@@ -1054,6 +1091,8 @@ int FTPLocalRecursiveFileList2(FTPCIPtr cip, FTPLineListPtr fileList, FTPFileInf
  */
 int FTPRemoteRecursiveFileList(FTPCIPtr, FTPLineListPtr, FTPFileInfoListPtr);
 int FTPRemoteRecursiveFileList1(FTPCIPtr, char *const, FTPFileInfoListPtr);
+int FTPRemoteFtwProc(const FtwInfoPtr ftwip);
+int FTPRemoteRecursiveFileList2(FTPCIPtr cip, char *const rdir, FTPFileInfoListPtr files);
 
 int FTPRebuildConnectionInfo(const FTPLIPtr lip, const FTPCIPtr cip);
 
@@ -1063,6 +1102,7 @@ int FTPFileExistsNlst(const FTPCIPtr cip, const char *const file);
 int FTPFileExists2(const FTPCIPtr cip, const char *const file, const int tryMDTM, const int trySIZE, const int tryMLST, const int trySTAT, const int tryNLST);
 
 void FTPGetDateStr(time_t t, const char *fmt, char *const ltstr1, const size_t ltstr1size, char *const gtstr1, const size_t gtstr1size);
+int BindToEphemeralPortNumber(const int sockfd, struct sockaddr_in *const addrp, const int ephemLo, const int ephemHi);
 int BufferGets(char *, size_t, int, char *, char **, char **, size_t);
 void DisposeFileInfoListContents(FTPFileInfoListPtr);
 void InitFileInfoList(FTPFileInfoListPtr);
