@@ -8,20 +8,26 @@
 #if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 #	pragma once
 #	define _CRT_SECURE_NO_WARNINGS 1
-#	pragma warning(disable : 4127)	// warning C4127: conditional expression is constant
-#	pragma warning(disable : 4100)	// warning C4100: 'lpReserved' : unreferenced formal parameter
-#	pragma warning(disable : 4514)	// warning C4514: unreferenced inline function has been removed
-#	pragma warning(disable : 4115)	// warning C4115: '_RPC_ASYNC_STATE' : named type definition in parentheses
-#	pragma warning(disable : 4201)	// warning C4201: nonstandard extension used : nameless struct/union
-#	pragma warning(disable : 4214)	// warning C4214: nonstandard extension used : bit field types other than int
-#	pragma warning(disable : 4115)	// warning C4115: 'IRpcStubBuffer' : named type definition in parentheses
+#	ifndef __MINGW32__
+#		pragma warning(disable : 4127)	// warning C4127: conditional expression is constant
+#		pragma warning(disable : 4100)	// warning C4100: 'lpReserved' : unreferenced formal parameter
+#		pragma warning(disable : 4514)	// warning C4514: unreferenced inline function has been removed
+#		pragma warning(disable : 4115)	// warning C4115: '_RPC_ASYNC_STATE' : named type definition in parentheses
+#		pragma warning(disable : 4201)	// warning C4201: nonstandard extension used : nameless struct/union
+#		pragma warning(disable : 4214)	// warning C4214: nonstandard extension used : bit field types other than int
+#		pragma warning(disable : 4115)	// warning C4115: 'IRpcStubBuffer' : named type definition in parentheses
+#	endif
 	/* Include "wincfg.h" in place of "config.h" */
 #	include "wincfg.h"
+	/* We now try for at least Windows 2000 compatibility (0x0500).
+	 * The code will still work on older systems, though.
+	 * Prior versions used 0x0400 instead.
+	 */
 #	ifndef WINVER
-#		define WINVER 0x0400
+#		define WINVER 0x0500
 #	endif
 #	ifndef _WIN32_WINNT
-#		define _WIN32_WINNT 0x0400
+#		define _WIN32_WINNT 0x0500
 #	endif
 #	include <windows.h>		/* includes <winsock2.h> if _WIN32_WINNT >= 0x400 */
 #	include <shlobj.h>
@@ -40,6 +46,9 @@
 #	include <sys/stat.h>
 #	include <sys/utime.h>
 #	include <fcntl.h>
+#	ifdef HAVE_FNMATCH_H
+#		include <fnmatch.h>
+#	endif
 #	define strcasecmp _stricmp
 #	define strncasecmp _strnicmp
 #	define strdup _strdup
@@ -147,6 +156,9 @@
 #	include <pwd.h>
 #	include <dirent.h>
 #	include <fcntl.h>
+#	ifdef HAVE_FNMATCH_H
+#		include <fnmatch.h>
+#	endif
 
 #	ifdef HAVE_NET_ERRNO_H
 #		include <net/errno.h>
@@ -265,16 +277,10 @@
 #ifndef Lseek
 #	if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 #		define Lseek(a,b,c) _lseeki64(a, (__int64) b, c)
-#	elif ((defined(_FILE_OFFSET_BITS)) && (_FILE_OFFSET_BITS > 32))
+#	elif (((defined(_FILE_OFFSET_BITS)) && (_FILE_OFFSET_BITS > 32)) || ((defined(SIZEOF_OFF_T)) && (SIZEOF_OFF_T > 4)))
 #		define Lseek(a,b,c) lseek(a, (off_t) b, c)
 #	elif ((defined(HAVE_LONG_LONG)) && (defined(_LARGEFILE64_SOURCE)) && (defined(HAVE_LSEEK64)))
 #		define Lseek(a,b,c) lseek64(a, (longest_int) b, c)
-#	elif defined(HAVE_LONG_LONG) && defined(HAVE_LLSEEK)
-#		if defined(LINUX) && (LINUX <= 23000)
-#			define Lseek(a,b,c) lseek(a, (off_t) b, c)
-#		else
-#			define Lseek(a,b,c) llseek(a, (longest_int) b, c)
-#		endif
 #	else
 #		define Lseek(a,b,c) lseek(a, (off_t) b, c)
 #	endif
@@ -353,4 +359,8 @@ extern ssize_t nsendmsg(int, const struct msghdr *, int);
 #include "util.h"
 #include "ftp.h"
 
+#ifdef DNSSEC_LOCAL_VALIDATION
+#include <validator/validator.h>
+#endif
+
 /* eof */
