@@ -1,6 +1,6 @@
 /* syshdrs.h
  *
- * Copyright (c) 1992-2004 by Mike Gleason.
+ * Copyright (c) 1992-2016 by Mike Gleason.
  * All rights reserved.
  * 
  */
@@ -8,14 +8,16 @@
 #if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 #	pragma once
 #	define _CRT_SECURE_NO_WARNINGS 1
-#	pragma warning(disable : 4127)	// warning C4127: conditional expression is constant
-#	pragma warning(disable : 4100)	// warning C4100: 'lpReserved' : unreferenced formal parameter
-#	pragma warning(disable : 4514)	// warning C4514: unreferenced inline function has been removed
-#	pragma warning(disable : 4115)	// warning C4115: '_RPC_ASYNC_STATE' : named type definition in parentheses
-#	pragma warning(disable : 4201)	// warning C4201: nonstandard extension used : nameless struct/union
-#	pragma warning(disable : 4214)	// warning C4214: nonstandard extension used : bit field types other than int
-#	pragma warning(disable : 4115)	// warning C4115: 'IRpcStubBuffer' : named type definition in parentheses
-#	pragma warning(disable : 4711)	// warning C4711: function selected for automatic inline expansion
+#	ifndef __MINGW32__
+#		pragma warning(disable : 4127)	// warning C4127: conditional expression is constant
+#		pragma warning(disable : 4100)	// warning C4100: 'lpReserved' : unreferenced formal parameter
+#		pragma warning(disable : 4514)	// warning C4514: unreferenced inline function has been removed
+#		pragma warning(disable : 4115)	// warning C4115: '_RPC_ASYNC_STATE' : named type definition in parentheses
+#		pragma warning(disable : 4201)	// warning C4201: nonstandard extension used : nameless struct/union
+#		pragma warning(disable : 4214)	// warning C4214: nonstandard extension used : bit field types other than int
+#		pragma warning(disable : 4115)	// warning C4115: 'IRpcStubBuffer' : named type definition in parentheses
+#		pragma warning(disable : 4711)	// warning C4711: function selected for automatic inline expansion
+#	endif
 #	define SELECT_TYPE_ARG1 int
 #	define SELECT_TYPE_ARG234 (fd_set *)
 #	define SELECT_TYPE_ARG5 (struct timeval *)
@@ -26,13 +28,18 @@
 #	define HAVE_STRSTR 1
 #	define HAVE_MEMMOVE 1
 #	define HAVE_LONG_FILE_NAMES 1
+	/* We now try for at least Windows 2000 compatibility (0x0500).
+	 * The code will still work on older systems, though.
+	 * Prior versions used 0x0400 instead.
+	 */
 #	ifndef WINVER
-#		define WINVER 0x0400
+#		define WINVER 0x0500
 #	endif
 #	ifndef _WIN32_WINNT
-#		define _WIN32_WINNT 0x0400
+#		define _WIN32_WINNT 0x0500
 #	endif
 #	include <windows.h>		/* includes <winsock2.h> if _WIN32_WINNT >= 0x400 */
+#	include <share.h>
 #	include <shlobj.h>
 #	include <process.h>
 #	include <direct.h>
@@ -56,6 +63,9 @@
 #	include <fcntl.h>
 #	include <signal.h>
 #	include <assert.h>
+#	ifdef HAVE_FNMATCH_H
+#		include <fnmatch.h>
+#	endif
 #	ifndef strcasecmp
 #		define strcasecmp _stricmp
 #		define strncasecmp _strnicmp
@@ -174,6 +184,9 @@
 #	include <time.h>
 #	include <pwd.h>
 #	include <fcntl.h>
+#	ifdef HAVE_FNMATCH_H
+#		include <fnmatch.h>
+#	endif
 #	if defined(HAVE_SYS_IOCTL_H) && defined(HAVE_TERMIOS_H)
 #		include <sys/ioctl.h>
 #		include <termios.h>
@@ -271,20 +284,10 @@
 #	define Lstat lstat
 #endif
 
-#if ((defined(_FILE_OFFSET_BITS)) && (_FILE_OFFSET_BITS > 32))
+#if (((defined(_FILE_OFFSET_BITS)) && (_FILE_OFFSET_BITS > 32)) || ((defined(SIZEOF_OFF_T)) && (SIZEOF_OFF_T > 4)))
 #	define Lseek(a,b,c) lseek(a, (off_t) b, c)
 #elif ((defined(HAVE_LONG_LONG)) && (defined(_LARGEFILE64_SOURCE)) && (defined(HAVE_LSEEK64)))
 #	define Lseek(a,b,c) lseek64(a, (longest_int) b, c)
-#elif defined(HAVE_LONG_LONG) && defined(HAVE_LLSEEK)
-#	if 1
-#		if defined(LINUX) && (LINUX <= 23000)
-#			define Lseek(a,b,c) lseek(a, (off_t) b, c)
-#		else
-#			define Lseek(a,b,c) llseek(a, (longest_int) b, c)
-#		endif
-#	else
-#		define Lseek(a,b,c) lseek(a, (off_t) b, c)
-#	endif
 #else
 #	define Lseek(a,b,c) lseek(a, (off_t) b, c)
 #endif
@@ -301,3 +304,8 @@
 #include <Strn.h>			/* Library header. */
 #include <sio.h>			/* Library header. */
 #include <ncftp.h>			/* Library header. */
+
+#ifdef DNSSEC_LOCAL_VALIDATION
+#include <validator/validator.h>
+#endif
+
